@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2018 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -51,6 +51,8 @@
 #include "shape.h"
 #include "measureunit.h"
 #include "casemap.h"
+#include "tries.h"
+#include "gender.h"
 
 
 /* const variable descriptor */
@@ -261,6 +263,8 @@ static PyObject *PyInit_icu(PyObject *m)
     _init_shape(m);
     _init_measureunit(m);
     _init_casemap(m);
+    _init_tries(m);
+    _init_gender(m);
 
     return m;
 }
