@@ -6,10 +6,10 @@
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions: 
+ * Software is furnished to do so, subject to the following conditions:
  *
  * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software. 
+ * in all copies or substantial portions of the Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -42,6 +42,30 @@ public:
     UObject *object;
 };
 
+class PythonReplaceable : public Replaceable {
+ public:
+  explicit PythonReplaceable(PyObject *self);
+  ~PythonReplaceable();
+
+  int32_t getLength() const override;
+#if U_ICU_VERSION_HEX < VERSION_HEX(59, 1, 0)
+  UChar getCharAt(int32_t offset) const override;
+#else
+  char16_t getCharAt(int32_t offset) const override;
+#endif
+  UChar32 getChar32At(int32_t offset) const override;
+
+  void extractBetween(
+      int32_t start, int32_t limit, UnicodeString &target) const override;
+  void handleReplaceBetween(
+      int32_t start, int32_t limit, const UnicodeString &text) override;
+  void copy(int32_t start, int32_t limit, int32_t dest) override;
+  UBool hasMetaData() const override;
+
+ private:
+  PyObject *self_;
+};
+
 struct UNone;
 typedef struct UNone UNone;
 
@@ -51,6 +75,7 @@ void t_uobject_dealloc(t_uobject *self);
 extern PyTypeObject UMemoryType_;
 extern PyTypeObject UObjectType_;
 extern PyTypeObject FormattableType_;
+extern PyTypeObject PythonReplaceableType_;
 
 PyObject *wrap_StringEnumeration(StringEnumeration *, int);
 PyObject *wrap_Formattable(Formattable *, int);
