@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2005-2014 Open Source Applications Foundation.
+ * Copyright (c) 2005-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -552,6 +552,17 @@ int abstract_init(PyObject *self, PyObject *args, PyObject *kwds)
     return -1;
 }
 
+PyObject *abstract_method(PyObject *self, PyObject *args)
+{
+    PyObject *err = Py_BuildValue("(sO)", "calling abstract method on",
+                                  self->ob_type);
+
+    PyErr_SetObject(PyExc_NotImplementedError, err);
+    Py_DECREF(err);
+
+    return NULL;
+}
+
 static PyObject *types;
 
 void registerType(PyTypeObject *type, classid id)
@@ -758,6 +769,56 @@ static UnicodeString *toUnicodeStringArray(PyObject *arg, int *len)
     return NULL;
 }
 
+static int *toIntArray(PyObject *arg, int *len)
+{
+    if (PySequence_Check(arg))
+    {
+        *len = (int) PySequence_Size(arg);
+        int *array = new int[*len + 1];
+
+        for (int i = 0; i < *len; i++) {
+            PyObject *obj = PySequence_GetItem(arg, i);
+
+#if PY_MAJOR_VERSION < 3
+            if (PyInt_Check(obj))
+            {
+                array[i] = PyInt_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+            else if (PyLong_Check(obj))
+            {
+                array[i] = PyLong_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+#else
+            if (PyLong_Check(obj))
+            {
+                array[i] = PyLong_AsLong(obj);
+                Py_DECREF(obj);
+
+                if (!PyErr_Occurred())
+                    continue;
+            }
+#endif
+
+            Py_DECREF(obj);
+            delete[] array;
+
+            return NULL;
+        }
+
+        return array;
+    }
+
+    return NULL;
+}
+
 static double *toDoubleArray(PyObject *arg, int *len)
 {
     if (PySequence_Check(arg))
@@ -1046,6 +1107,23 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
                 break;
             return -1;
 
+          case 'H':           /* array of int */
+            if (PySequence_Check(arg))
+            {
+                if (PySequence_Length(arg) > 0)
+                {
+                    PyObject *obj = PySequence_GetItem(arg, 0);
+                    int ok = (PyInt_Check(obj) ||
+                              PyLong_Check(obj));
+                    Py_DECREF(obj);
+                    if (ok)
+                        break;
+                }
+                else
+                    break;
+            }
+            return -1;
+
           case 'L':           /* PY_LONG_LONG */
             if (PyLong_Check(arg) || PyInt_Check(arg))
                 break;
@@ -1354,6 +1432,16 @@ int _parseArgs(PyObject **args, int count, const char *types, ...)
               break;
           }
 
+          case 'H':           /* array of int */
+          {
+              int **array = va_arg(list, int **);
+              int *len = va_arg(list, int *);
+              *array = toIntArray(arg, len);
+              if (!*array)
+                  return -1;
+              break;
+          }
+
           case 'L':           /* PY_LONG_LONG */
           {
               PY_LONG_LONG *l = va_arg(list, PY_LONG_LONG *);
@@ -1402,7 +1490,7 @@ int isUnicodeString(PyObject *arg)
             ISINSTANCE(((t_uobject *) arg)->object, UnicodeString));
 }
 
-int32_t toUChar32(UnicodeString& u, UChar32 *c, UErrorCode& status)
+int32_t toUChar32(UnicodeString &u, UChar32 *c, UErrorCode &status)
 {
 #if U_ICU_VERSION_HEX >= 0x04020000
     return u.toUTF32(c, 1, status);
