@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2005-2018 Open Source Applications Foundation.
+ * Copyright (c) 2005-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -160,6 +160,7 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/uversion.h>
 #include <unicode/ushape.h>
 #include <unicode/ucasemap.h>
+#include <unicode/udata.h>
 
 #define VERSION_HEX(major, minor, patch) \
   (((major) << 24) | ((minor) << 16) | ((patch) << 8))
@@ -182,8 +183,8 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/uspoof.h>
 #include <unicode/tmunit.h>
 #include <unicode/tmutamt.h>
-#include <unicode/stringpiece.h>
 #include <unicode/numsys.h>
+
 #endif
 
 #if U_ICU_VERSION_HEX >= 0x04040000
@@ -197,10 +198,17 @@ typedef intintobjargproc ssizessizeobjargproc;
 
 #if U_ICU_VERSION_HEX >= 0x04080000
 #include <unicode/alphaindex.h>
+#include <unicode/stringtriebuilder.h>
+#include <unicode/bytestriebuilder.h>
+#include <unicode/bytestrie.h>
+#include <unicode/ucharstriebuilder.h>
+#include <unicode/ucharstrie.h>
+#include <unicode/appendable.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
 #include <unicode/listformatter.h>
+#include <unicode/gender.h>
 #endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
@@ -229,6 +237,14 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/stringoptions.h>
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+#include <unicode/localebuilder.h>
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+#include <unicode/localematcher.h>
+#endif
+
 #if U_ICU_VERSION_HEX < 0x04060000
 
 typedef UClassID classid;
@@ -262,7 +278,7 @@ typedef const char *classid;
 
 U_NAMESPACE_USE
 
-/* lifted from ustrenum.h */
+/* lifted from common/ustrenum.h */
 
 U_NAMESPACE_BEGIN
 class U_COMMON_API UStringEnumeration : public StringEnumeration {
@@ -326,6 +342,7 @@ EXPORT UnicodeString *PyObject_AsUnicodeString(PyObject *object);
 EXPORT UDate PyObject_AsUDate(PyObject *object);
 
 int abstract_init(PyObject *self, PyObject *args, PyObject *kwds);
+PyObject *abstract_method(PyObject *self, PyObject *args);
 
 // helper class, to allow argument parsing with proper cleanup
 class charsArg {
