@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2011 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -259,22 +259,35 @@ class t_rulebasedbreakiterator : public _wrapper {
 public:
     RuleBasedBreakIterator *object;
     PyObject *text;  /* used by inherited BreakIterator.setText() */
+    PyObject *binaryRules;
 };
 
 static int t_rulebasedbreakiterator_init(t_rulebasedbreakiterator *self,
                                          PyObject *args, PyObject *kwds);
 static PyObject *t_rulebasedbreakiterator_getRules(t_rulebasedbreakiterator *self, PyObject *args);
 static PyObject *t_rulebasedbreakiterator_getRuleStatus(t_rulebasedbreakiterator *self);
+#if U_ICU_VERSION_HEX >= 0x04080000
+static PyObject *t_rulebasedbreakiterator_getBinaryRules(t_rulebasedbreakiterator *self);
+#endif
 
 static PyMethodDef t_rulebasedbreakiterator_methods[] = {
     DECLARE_METHOD(t_rulebasedbreakiterator, getRules, METH_VARARGS),
     DECLARE_METHOD(t_rulebasedbreakiterator, getRuleStatus, METH_NOARGS),
+#if U_ICU_VERSION_HEX >= 0x04080000
+    DECLARE_METHOD(t_rulebasedbreakiterator, getBinaryRules, METH_NOARGS),
+#endif
     { NULL, NULL, 0, NULL }
 };
 
+static void t_rulebasedbreakiterator_dealloc(t_rulebasedbreakiterator *self)
+{
+    Py_CLEAR(self->binaryRules);
+    t_breakiterator_dealloc((t_breakiterator *) self);
+}
+
 DECLARE_TYPE(RuleBasedBreakIterator, t_rulebasedbreakiterator,
              BreakIterator, RuleBasedBreakIterator,
-             t_rulebasedbreakiterator_init, NULL);
+             t_rulebasedbreakiterator_init, t_rulebasedbreakiterator_dealloc);
 
 /* DictionaryBasedBreakIterator */
 
@@ -400,31 +413,7 @@ static PyObject *t_forwardcharacteriterator_iter(t_forwardcharacteriterator *sel
     Py_RETURN_SELF();
 }
 
-static PyObject *t_forwardcharacteriterator_richcmp(t_forwardcharacteriterator *self, PyObject *arg, int op)
-{
-    int b = 0;
-    ForwardCharacterIterator *i;
-
-    if (!parseArg(arg, "P", TYPE_ID(ForwardCharacterIterator), &i))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *i;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
+DEFINE_RICHCMP(ForwardCharacterIterator, t_forwardcharacteriterator);
 
 
 /* CharacterIterator */
@@ -1018,34 +1007,7 @@ static PyObject *t_breakiterator_iter_next(t_breakiterator *self)
     return PyInt_FromLong(n);
 }
 
-
-
-static PyObject *t_breakiterator_richcmp(t_breakiterator *self,
-                                         PyObject *arg, int op)
-{
-    int b = 0;
-    BreakIterator *i;
-
-    if (!parseArg(arg, "P", TYPE_ID(BreakIterator), &i))
-    {
-        switch (op) {
-          case Py_EQ:
-          case Py_NE:
-            b = *self->object == *i;
-            if (op == Py_EQ)
-                Py_RETURN_BOOL(b);
-            Py_RETURN_BOOL(!b);
-          case Py_LT:
-          case Py_LE:
-          case Py_GT:
-          case Py_GE:
-            PyErr_SetNone(PyExc_NotImplementedError);
-            return NULL;
-        }
-    }
-
-    return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg);
-}
+DEFINE_RICHCMP(BreakIterator, t_breakiterator);
 
 
 /* RuleBasedBreakIterator */
@@ -1055,6 +1017,7 @@ static int t_rulebasedbreakiterator_init(t_rulebasedbreakiterator *self,
 {
     UnicodeString *u, _u;
     charsArg path, name;
+    PyObject *buffer;
 
     switch (PyTuple_Size(args)) {
       case 0:
@@ -1062,6 +1025,28 @@ static int t_rulebasedbreakiterator_init(t_rulebasedbreakiterator *self,
         self->flags = T_OWNED;
         break;
       case 1:
+#if U_ICU_VERSION_HEX >= 0x04080000
+        if (!parseArgs(args, "C", &buffer))
+        {
+            UErrorCode status = U_ZERO_ERROR;
+            RuleBasedBreakIterator *iterator = new RuleBasedBreakIterator(
+                (const uint8_t *) PyBytes_AS_STRING(buffer),
+                (uint32_t) PyBytes_GET_SIZE(buffer), status);
+
+            // since "C" and "S" overlap fall through to "S" case if failure
+            if (U_SUCCESS(status))
+            {
+                self->object = iterator;
+                self->flags = T_OWNED;
+                // keep buffer for lifetime of object
+                self->binaryRules = buffer;
+                Py_INCREF(buffer);
+                break;
+            } else {
+              delete iterator;
+            }
+        }
+#endif
         if (!parseArgs(args, "S", &u, &_u))
         {
             RuleBasedBreakIterator *iterator;
@@ -1142,7 +1127,15 @@ static PyObject *t_rulebasedbreakiterator_getRuleStatus(t_rulebasedbreakiterator
     return PyInt_FromLong(self->object->getRuleStatus());
 }
 
+#if U_ICU_VERSION_HEX >= 0x04080000
+static PyObject *t_rulebasedbreakiterator_getBinaryRules(t_rulebasedbreakiterator *self)
+{
+    uint32_t length = 0;
+    const uint8_t *data = self->object->getBinaryRules(length);
 
+    return PyBytes_FromStringAndSize((char *) data, (int) length);
+}
+#endif
 
 /* DictionaryBasedBreakIterator */
 
