@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2014 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -50,6 +50,11 @@ DECLARE_CONSTANTS_TYPE(UMeasurementSystem);
 DECLARE_CONSTANTS_TYPE(URegionType);
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+DECLARE_CONSTANTS_TYPE(ULocMatchFavorSubtag);
+DECLARE_CONSTANTS_TYPE(ULocMatchDemotion);
+#endif
+
 
 /* Locale */
 
@@ -74,6 +79,9 @@ static PyObject *t_locale_getDisplayCountry(t_locale *self, PyObject *args);
 static PyObject *t_locale_getDisplayVariant(t_locale *self, PyObject *args);
 static PyObject *t_locale_getDisplayName(t_locale *self, PyObject *args);
 static PyObject *t_locale_createKeywords(t_locale *self);
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+static PyObject *t_locale_createUnicodeKeywords(t_locale *self);
+#endif
 static PyObject *t_locale_getKeywordValue(t_locale *self, PyObject *arg);
 #if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
 static PyObject *t_locale_setKeywordValue(t_locale *self, PyObject *args);
@@ -137,6 +145,9 @@ static PyMethodDef t_locale_methods[] = {
     DECLARE_METHOD(t_locale, getDisplayVariant, METH_VARARGS),
     DECLARE_METHOD(t_locale, getDisplayName, METH_VARARGS),
     DECLARE_METHOD(t_locale, createKeywords, METH_NOARGS),
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+    DECLARE_METHOD(t_locale, createUnicodeKeywords, METH_NOARGS),
+#endif
     DECLARE_METHOD(t_locale, getKeywordValue, METH_O),
 #if U_ICU_VERSION_HEX >= VERSION_HEX(49, 0, 0)
     DECLARE_METHOD(t_locale, setKeywordValue, METH_VARARGS),
@@ -356,23 +367,213 @@ DECLARE_TYPE(Region, t_region, UObject, const Region, abstract_init, NULL);
 
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+
+/* LocaleBuilder */
+
+class t_localebuilder : public _wrapper {
+public:
+    LocaleBuilder *object;
+};
+
+static int t_localebuilder_init(t_localebuilder *self, PyObject *args, PyObject *kwds);
+static PyObject *t_localebuilder_setLocale(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setLanguageTag(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setLanguage(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setScript(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setRegion(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setVariant(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_addUnicodeLocaleAttribute(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_removeUnicodeLocaleAttribute(t_localebuilder *self, PyObject *arg);
+static PyObject *t_localebuilder_setExtension(t_localebuilder *self, PyObject *args);
+static PyObject *t_localebuilder_setUnicodeLocaleKeyword(t_localebuilder *self, PyObject *args);
+static PyObject *t_localebuilder_clear(t_localebuilder *self);
+static PyObject *t_localebuilder_clearExtensions(t_localebuilder *self);
+static PyObject *t_localebuilder_build(t_localebuilder *self);
+
+static PyMethodDef t_localebuilder_methods[] = {
+    DECLARE_METHOD(t_localebuilder, setLocale, METH_O),
+    DECLARE_METHOD(t_localebuilder, setLanguageTag, METH_O),
+    DECLARE_METHOD(t_localebuilder, setLanguage, METH_O),
+    DECLARE_METHOD(t_localebuilder, setScript, METH_O),
+    DECLARE_METHOD(t_localebuilder, setRegion, METH_O),
+    DECLARE_METHOD(t_localebuilder, setVariant, METH_O),
+    DECLARE_METHOD(t_localebuilder, addUnicodeLocaleAttribute, METH_O),
+    DECLARE_METHOD(t_localebuilder, removeUnicodeLocaleAttribute, METH_O),
+    DECLARE_METHOD(t_localebuilder, setExtension, METH_VARARGS),
+    DECLARE_METHOD(t_localebuilder, setUnicodeLocaleKeyword, METH_VARARGS),
+    DECLARE_METHOD(t_localebuilder, clear, METH_NOARGS),
+    DECLARE_METHOD(t_localebuilder, clearExtensions, METH_NOARGS),
+    DECLARE_METHOD(t_localebuilder, build, METH_NOARGS),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_TYPE(LocaleBuilder, t_localebuilder, UObject, LocaleBuilder,
+             t_localebuilder_init, NULL);
+
+#endif
+
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+
+using LocaleMatcherBuilder = LocaleMatcher::Builder;
+using LocaleMatcherResult = LocaleMatcher::Result;
+
+/* LocaleMatcherBuilder */
+
+class t_localematcherbuilder : public _wrapper {
+public:
+  LocaleMatcherBuilder *object;
+};
+
+static int t_localematcherbuilder_init(t_localematcherbuilder *self, PyObject *args, PyObject *kwds);
+static PyObject *t_localematcherbuilder_setSupportedLocalesFromListString(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_setSupportedLocales(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_addSupportedLocale(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_setDefaultLocale(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_setFavorSubtag(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_setDemotionPerDesiredLocale(t_localematcherbuilder *self, PyObject *arg);
+static PyObject *t_localematcherbuilder_build(t_localematcherbuilder *self);
+
+static PyMethodDef t_localematcherbuilder_methods[] = {
+    DECLARE_METHOD(t_localematcherbuilder, setSupportedLocalesFromListString, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, setSupportedLocales, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, addSupportedLocale, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, setDefaultLocale, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, setFavorSubtag, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, setDemotionPerDesiredLocale, METH_O),
+    DECLARE_METHOD(t_localematcherbuilder, build, METH_NOARGS),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_TYPE(LocaleMatcherBuilder, t_localematcherbuilder, UMemory,
+             LocaleMatcherBuilder, t_localematcherbuilder_init, NULL);
+
+/* LocaleMatcherResult */
+
+class t_localematcherresult : public _wrapper {
+public:
+  LocaleMatcherResult *object;
+};
+
+static PyObject *t_localematcherresult_getDesiredLocale(t_localematcherresult *self);
+static PyObject *t_localematcherresult_getSupportedLocale(t_localematcherresult *self);
+static PyObject *t_localematcherresult_getDesiredIndex(t_localematcherresult *self);
+static PyObject *t_localematcherresult_getSupportedIndex(t_localematcherresult *self);
+static PyObject *t_localematcherresult_makeResolvedLocale(t_localematcherresult *self);
+
+static PyMethodDef t_localematcherresult_methods[] = {
+    DECLARE_METHOD(t_localematcherresult, getDesiredLocale, METH_NOARGS),
+    DECLARE_METHOD(t_localematcherresult, getSupportedLocale, METH_NOARGS),
+    DECLARE_METHOD(t_localematcherresult, getDesiredIndex, METH_NOARGS),
+    DECLARE_METHOD(t_localematcherresult, getSupportedIndex, METH_NOARGS),
+    DECLARE_METHOD(t_localematcherresult, makeResolvedLocale, METH_NOARGS),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_BY_VALUE_TYPE(LocaleMatcherResult, t_localematcherresult, UMemory,
+                      LocaleMatcherResult, abstract_init);
+
+/* LocaleMatcher */
+
+class t_localematcher : public _wrapper {
+public:
+  LocaleMatcher *object;
+};
+
+static PyObject *t_localematcher_getBestMatch(t_localematcher *self, PyObject *arg);
+static PyObject *t_localematcher_getBestMatchForListString(t_localematcher *self, PyObject *arg);
+static PyObject *t_localematcher_getBestMatchResult(t_localematcher *self, PyObject *arg);
+
+static PyMethodDef t_localematcher_methods[] = {
+    DECLARE_METHOD(t_localematcher, getBestMatch, METH_O),
+    DECLARE_METHOD(t_localematcher, getBestMatchForListString, METH_O),
+    DECLARE_METHOD(t_localematcher, getBestMatchResult, METH_O),
+    { NULL, NULL, 0, NULL }
+};
+
+DECLARE_BY_VALUE_TYPE(LocaleMatcher, t_localematcher, UMemory,
+                      LocaleMatcher, abstract_init);
+
+#endif
+
 
 /* Locale */
 
 static int t_locale_init(t_locale *self, PyObject *args, PyObject *kwds)
 {
-    charsArg language, country, variant;
+    charsArg language, country, variant, keywords;
     int lcid, len;
 
+    if (PyTuple_Size(args) < 4 && kwds != NULL)
+    {
+        PyObject *items = PyDict_Items(kwds);
+        int len = PySequence_Fast_GET_SIZE(items);
+
+        if (len > 0)
+        {
+            PyObject *eq = PyUnicode_FromString("=");
+            PyObject *all = NULL;
+
+            if (len > 1)
+            {
+                PyObject *pairs = PyTuple_New(len);
+
+                for (int i = 0; i < len; ++i)
+                {
+                    PyObject *pair = PyUnicode_Join(
+                        eq, PySequence_Fast_GET_ITEM(items, i));
+
+                    if (pair == NULL)
+                    {
+                        Py_DECREF(pairs);
+                        Py_DECREF(eq);
+                        Py_DECREF(items);
+
+                        return -1;
+                    }
+
+                    PyTuple_SET_ITEM(pairs, i, pair);
+                }
+
+                PyObject *sm = PyUnicode_FromString(";");
+                all = PyUnicode_Join(sm, pairs);
+
+                Py_DECREF(sm);
+                Py_DECREF(pairs);
+            }
+            else
+                all = PyUnicode_Join(eq, PySequence_Fast_GET_ITEM(items, 0));
+
+            Py_DECREF(eq);
+            Py_DECREF(items);
+
+            if (all == NULL)
+                return -1;
+
+            PyObject *bytes = PyUnicode_AsASCIIString(all);
+
+            Py_DECREF(all);
+            if (bytes == NULL)
+                return -1;
+
+            keywords.own(bytes);
+        }
+        else
+            Py_DECREF(items);
+    }
+
     switch (PyTuple_Size(args)) {
       case 0:
-        self->object = new Locale();
+        self->object = new Locale(NULL, NULL, NULL,
+                                  kwds != NULL ? keywords.c_str() : NULL);
         self->flags = T_OWNED;
         break;
       case 1:
         if (!parseArgs(args, "n", &language))
         {
-            self->object = new Locale(language);
+            self->object = new Locale(language, NULL, NULL,
+                                      kwds != NULL ? keywords.c_str() : NULL);
             self->flags = T_OWNED;
             break;
         }
@@ -384,7 +585,8 @@ static int t_locale_init(t_locale *self, PyObject *args, PyObject *kwds)
                 lcid, code, sizeof(code), &status));
             if ((size_t) len < sizeof(code))
             {
-                self->object = new Locale(code);
+                self->object = new Locale(
+                    code, NULL, NULL, kwds != NULL ? keywords.c_str() : NULL);
                 self->flags = T_OWNED;
             }
             break;
@@ -394,7 +596,8 @@ static int t_locale_init(t_locale *self, PyObject *args, PyObject *kwds)
       case 2:
         if (!parseArgs(args, "nn", &language, &country))
         {
-            self->object = new Locale(language, country);
+            self->object = new Locale(language, country, NULL,
+                                      kwds != NULL ? keywords.c_str() : NULL);
             self->flags = T_OWNED;
             break;
         }
@@ -403,7 +606,17 @@ static int t_locale_init(t_locale *self, PyObject *args, PyObject *kwds)
       case 3:
         if (!parseArgs(args, "nnn", &language, &country, &variant))
         {
-            self->object = new Locale(language, country, variant);
+            self->object = new Locale(language, country, variant,
+                                      kwds != NULL ? keywords.c_str() : NULL);
+            self->flags = T_OWNED;
+            break;
+        }
+        PyErr_SetArgsError((PyObject *) self, "__init__", args);
+        return -1;
+      case 4:
+        if (!parseArgs(args, "nnnn", &language, &country, &variant, &keywords))
+        {
+            self->object = new Locale(language, country, variant, keywords);
             self->flags = T_OWNED;
             break;
         }
@@ -647,6 +860,17 @@ static PyObject *t_locale_createKeywords(t_locale *self)
     return wrap_StringEnumeration(se, T_OWNED);
 }
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+static PyObject *t_locale_createUnicodeKeywords(t_locale *self)
+{
+    StringEnumeration *se;
+    STATUS_CALL(se = self->object->createUnicodeKeywords(status));
+
+    return wrap_StringEnumeration(se, T_OWNED);
+}
+
+#endif
+
 static PyObject *t_locale_getKeywordValue(t_locale *self, PyObject *arg)
 {
     charsArg name;
@@ -708,7 +932,8 @@ static PyObject *t_locale_setToBogus(t_locale *self)
     Py_RETURN_NONE;
 }
 
-#if U_ICU_VERSION_HEX >= 0x04000000
+#if U_ICU_VERSION_HEX >= 0x04000000 && U_ICU_VERSION_HEX < VERSION_HEX(63, 0, 0)
+
 static PyObject *t_locale_addLikelySubtags(t_locale *self)
 {
     char maximized[128];
@@ -732,6 +957,21 @@ static PyObject *t_locale_minimizeSubtags(t_locale *self)
 
     return PyString_FromStringAndSize(minimized, size);
 }
+
+#elif U_ICU_VERSION_HEX >= VERSION_HEX(63, 0, 0)
+
+static PyObject *t_locale_addLikelySubtags(t_locale *self)
+{
+    STATUS_CALL(self->object->addLikelySubtags(status));
+    Py_RETURN_SELF();
+}
+
+static PyObject *t_locale_minimizeSubtags(t_locale *self)
+{
+    STATUS_CALL(self->object->minimizeSubtags(status));
+    Py_RETURN_SELF();
+}
+
 #endif
 
 
@@ -916,7 +1156,7 @@ static PyObject *t_locale_forLanguageTag(PyTypeObject *type, PyObject *arg)
 
     if (!parseArg(arg, "n", &name))
     {
-        STATUS_CALL(locale = Locale::forLanguageTag(StringPiece(name), status));
+        STATUS_CALL(locale = Locale::forLanguageTag(name.c_str(), status));
         return wrap_Locale(locale);
     }
 
@@ -1002,6 +1242,8 @@ static long t_locale_hash(t_locale *self)
     return (long) self->object->hashCode();
 }
 
+DEFINE_RICHCMP(Locale, t_locale);
+
 
 /* ResourceBundle */
 
@@ -1737,10 +1979,399 @@ static PyObject *t_region_str(t_region *self)
 #endif
 
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+
+/* LocaleBuilder */
+
+static int t_localebuilder_init(t_localebuilder *self,
+                                PyObject *args, PyObject *kwds)
+{
+    switch (PyTuple_Size(args)) {
+      case 0:
+        self->object = new LocaleBuilder();
+        self->flags = T_OWNED;
+        break;
+      default:
+        PyErr_SetArgsError((PyObject *) self, "__init__", args);
+        return -1;
+    }
+
+    if (self->object)
+        return 0;
+
+    return -1;
+}
+
+static PyObject *t_localebuilder_setLocale(t_localebuilder *self, PyObject *arg)
+{
+    const Locale *locale;
+
+    if (!parseArg(arg, "P", TYPE_CLASSID(Locale), &locale))
+    {
+        self->object->setLocale(*locale);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setLocale", arg);
+}
+
+#define applyLBStringPiece(setter) \
+    static PyObject *t_localebuilder_ ## setter(t_localebuilder *self, \
+                                                PyObject *arg)         \
+    {                                                                  \
+        charsArg carg;                                                 \
+        if (!parseArg(arg, "n", &carg))                                \
+        {                                                              \
+            self->object->setter(carg.c_str());                        \
+            Py_RETURN_SELF();                                          \
+        }                                                              \
+        return PyErr_SetArgsError((PyObject *) self, #setter, arg);    \
+    }
+
+applyLBStringPiece(setLanguageTag)
+applyLBStringPiece(setLanguage)
+applyLBStringPiece(setScript)
+applyLBStringPiece(setRegion)
+applyLBStringPiece(setVariant)
+applyLBStringPiece(addUnicodeLocaleAttribute)
+applyLBStringPiece(removeUnicodeLocaleAttribute)
+
+static PyObject *t_localebuilder_setExtension(t_localebuilder *self,
+                                              PyObject *args)
+{
+    charsArg key, value;
+
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "nn", &key, &value) && strlen(key.c_str()) == 1)
+        {
+            self->object->setExtension(key.c_str()[0], value.c_str());
+            Py_RETURN_SELF();
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setExtension", args);
+}
+
+static PyObject *t_localebuilder_setUnicodeLocaleKeyword(t_localebuilder *self,
+                                                         PyObject *args)
+{
+    charsArg key, type;
+
+    switch (PyTuple_Size(args)) {
+      case 2:
+        if (!parseArgs(args, "nn", &key, &type))
+        {
+            self->object->setUnicodeLocaleKeyword(key.c_str(), type.c_str());
+            Py_RETURN_SELF();
+        }
+        break;
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setUnicodeLocaleKeyword", args);
+}
+
+static PyObject *t_localebuilder_clear(t_localebuilder *self)
+{
+    self->object->clear();
+    Py_RETURN_SELF();
+}
+
+static PyObject *t_localebuilder_clearExtensions(t_localebuilder *self)
+{
+    self->object->clearExtensions();
+    Py_RETURN_SELF();
+}
+
+static PyObject *t_localebuilder_build(t_localebuilder *self)
+{
+    Locale locale;
+    STATUS_CALL(locale = self->object->build(status));
+
+    return wrap_Locale(locale);
+}
+
+#endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+
+// takes ownership of locales pointer array
+class LocaleIterator : public Locale::Iterator {
+  public:
+    LocaleIterator(Locale **locales, int len) :
+        locales_(locales), len_(len),
+        current_(0)
+    {}
+
+    ~LocaleIterator()
+    {
+        free(locales_);  // allocated with calloc in pl2cpa in common.cpp
+    }
+
+    UBool hasNext() const override
+    {
+        return current_ < len_;
+    }
+
+    const Locale &next() override
+    {
+        return *locales_[current_++];
+    }
+
+  private:
+    Locale **locales_;
+    int len_;
+    int current_;
+};
+
+/* LocaleMatcherBuilder */
+
+static int t_localematcherbuilder_init(t_localematcherbuilder *self,
+                                       PyObject *args, PyObject *kwds)
+{
+    switch (PyTuple_Size(args)) {
+      case 0:
+        self->object = new LocaleMatcherBuilder();
+        self->flags = T_OWNED;
+        break;
+      default:
+        PyErr_SetArgsError((PyObject *) self, "__init__", args);
+        return -1;
+    }
+
+    if (self->object)
+        return 0;
+
+    return -1;
+}
+
+static PyObject *t_localematcherbuilder_setSupportedLocalesFromListString(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    charsArg locales;
+
+    if (!parseArg(arg, "n", &locales))
+    {
+        self->object->setSupportedLocalesFromListString(locales.c_str());
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self,
+                              "setSupportedLocalesFromListString", arg);
+}
+
+static PyObject *t_localematcherbuilder_setSupportedLocales(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    Locale **locales;
+    int len;
+
+    if (!parseArg(arg, "Q", TYPE_CLASSID(Locale), &locales, &len,
+                  TYPE_CLASSID(Locale)))
+    {
+        LocaleIterator it(locales, len);
+
+        self->object->setSupportedLocales(it);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setSupportedLocales", arg);
+}
+
+static PyObject *t_localematcherbuilder_addSupportedLocale(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    Locale *locale;
+
+    if (!parseArg(arg, "P", TYPE_CLASSID(Locale), &locale))
+    {
+        self->object->addSupportedLocale(*locale);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "addSupportedLocale", arg);
+}
+
+static PyObject *t_localematcherbuilder_setDefaultLocale(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    Locale *locale;
+
+    if (!parseArg(arg, "P", TYPE_CLASSID(Locale), &locale))
+    {
+        self->object->setDefaultLocale(locale);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setDefaultLocale", arg);
+}
+
+static PyObject *t_localematcherbuilder_setFavorSubtag(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    int option;
+
+    if (!parseArg(arg, "n", &option))
+    {
+        self->object->setFavorSubtag((ULocMatchFavorSubtag) option);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "setFavorSubtag", arg);
+}
+
+static PyObject *t_localematcherbuilder_setDemotionPerDesiredLocale(
+    t_localematcherbuilder *self, PyObject *arg)
+{
+    int option;
+
+    if (!parseArg(arg, "n", &option))
+    {
+        self->object->setDemotionPerDesiredLocale((ULocMatchDemotion) option);
+        Py_RETURN_SELF();
+    }
+
+    return PyErr_SetArgsError((PyObject *) self,
+                              "setDemotionPerDesiredLocale", arg);
+}
+
+static PyObject *t_localematcherbuilder_build(t_localematcherbuilder *self)
+{
+    STATUS_RESULT_CALL(LocaleMatcher matcher = self->object->build(status),
+                       return wrap_LocaleMatcher(std::move(matcher)));
+}
+
+
+/* LocaleMatcherResult */
+
+static PyObject *t_localematcherresult_getDesiredLocale(
+    t_localematcherresult *self)
+{
+    const Locale *locale = self->object->getDesiredLocale();
+
+    if (locale != nullptr)
+        return wrap_Locale(*locale);
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *t_localematcherresult_getSupportedLocale(
+    t_localematcherresult *self)
+{
+    const Locale *locale = self->object->getSupportedLocale();
+
+    if (locale != nullptr)
+        return wrap_Locale(*locale);
+
+    Py_RETURN_NONE;
+}
+
+static PyObject *t_localematcherresult_getDesiredIndex(
+    t_localematcherresult *self)
+{
+    return PyInt_FromLong(self->object->getDesiredIndex());
+}
+
+static PyObject *t_localematcherresult_getSupportedIndex(
+    t_localematcherresult *self)
+{
+    return PyInt_FromLong(self->object->getSupportedIndex());
+}
+
+static PyObject *t_localematcherresult_makeResolvedLocale(
+    t_localematcherresult *self)
+{
+    STATUS_RESULT_CALL(Locale locale = self->object->makeResolvedLocale(status),
+                       return wrap_Locale(locale));
+}
+
+
+/* LocaleMatcher */
+
+static PyObject *t_localematcher_getBestMatch(t_localematcher *self,
+                                              PyObject *arg) {
+    const Locale *locale;
+    Locale **locales;
+    int len;
+
+    if (!parseArg(arg, "P", TYPE_CLASSID(Locale), &locale))
+    {
+        const Locale *result;
+        STATUS_CALL(result = self->object->getBestMatch(*locale, status));
+
+        return wrap_Locale(*result);
+    }
+
+    if (!parseArg(arg, "Q", TYPE_CLASSID(Locale), &locales, &len,
+                  TYPE_CLASSID(Locale)))
+    {
+        LocaleIterator it(locales, len);
+        const Locale *result;
+
+        STATUS_CALL(result = self->object->getBestMatch(it, status));
+
+        return wrap_Locale(*result);
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "getBestMatch", arg);
+}
+
+static PyObject *t_localematcher_getBestMatchForListString(
+    t_localematcher *self, PyObject *arg)
+{
+    charsArg locales;
+
+    if (!parseArg(arg, "n", &locales))
+    {
+        const Locale *result;
+
+        STATUS_CALL(result = self->object->getBestMatchForListString(
+            locales.c_str(), status));
+
+        return wrap_Locale(*result);
+    }
+
+    return PyErr_SetArgsError((PyObject *) self,
+                              "getBestMatchForListString", arg);
+}
+
+static PyObject *t_localematcher_getBestMatchResult(
+    t_localematcher *self, PyObject *arg)
+{
+    Locale *locale;
+    Locale **locales;
+    int len;
+
+    if (!parseArg(arg, "P", TYPE_CLASSID(Locale), &locale))
+    {
+        STATUS_RESULT_CALL(
+            LocaleMatcherResult result = self->object->getBestMatchResult(
+                *locale, status),
+            return wrap_LocaleMatcherResult(std::move(result)));
+    }
+
+    if (!parseArg(arg, "Q", TYPE_CLASSID(Locale), &locales, &len,
+                  TYPE_CLASSID(Locale)))
+    {
+        LocaleIterator it(locales, len);
+
+        STATUS_RESULT_CALL(
+            LocaleMatcherResult result = self->object->getBestMatchResult(
+                it, status),
+            return wrap_LocaleMatcherResult(std::move(result)));
+    }
+
+    return PyErr_SetArgsError((PyObject *) self, "getBestMatchResult", arg);
+}
+
+#endif
+
 void _init_locale(PyObject *m)
 {
     LocaleType_.tp_str = (reprfunc) t_locale_str;
     LocaleType_.tp_hash = (hashfunc) t_locale_hash;
+    LocaleType_.tp_richcompare = (richcmpfunc) t_locale_richcmp;
     ResourceBundleType_.tp_iter = (getiterfunc) t_resourcebundle_iter;
     ResourceBundleType_.tp_iternext = (iternextfunc) t_resourcebundle_next;
     ResourceBundleType_.tp_str = (reprfunc) t_resourcebundle_str;
@@ -1760,6 +2391,21 @@ void _init_locale(PyObject *m)
     REGISTER_TYPE(Region, m);
     INSTALL_CONSTANTS_TYPE(URegionType, m);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    REGISTER_TYPE(LocaleBuilder, m);
+#endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+    INSTALL_STRUCT(LocaleMatcherBuilder, m);
+    INSTALL_STRUCT(LocaleMatcherResult, m);
+    INSTALL_STRUCT(LocaleMatcher, m);
+    INSTALL_CONSTANTS_TYPE(ULocMatchFavorSubtag, m);
+    INSTALL_CONSTANTS_TYPE(ULocMatchDemotion, m);
+
+    PyDict_SetItemString(LocaleMatcherType_.tp_dict, "Builder",
+                         (PyObject *) &LocaleMatcherBuilderType_);
+    PyDict_SetItemString(LocaleMatcherType_.tp_dict, "Result",
+                         (PyObject *) &LocaleMatcherResultType_);
+#endif
 
     INSTALL_ENUM(ULocDataLocaleType, "ACTUAL_LOCALE", ULOC_ACTUAL_LOCALE);
     INSTALL_ENUM(ULocDataLocaleType, "VALID_LOCALE", ULOC_VALID_LOCALE);
@@ -1809,4 +2455,12 @@ void _init_locale(PyObject *m)
     INSTALL_ENUM(URegionType, "GROUPING", URGN_GROUPING);
     INSTALL_ENUM(URegionType, "DEPRECATED", URGN_DEPRECATED);
 #endif
+
+#if U_ICU_VERSION_HEX >= VERSION_HEX(65, 0, 0)
+    INSTALL_ENUM(ULocMatchFavorSubtag, "LANGUAGE", ULOCMATCH_FAVOR_LANGUAGE);
+    INSTALL_ENUM(ULocMatchFavorSubtag, "SCRIPT", ULOCMATCH_FAVOR_SCRIPT);
+
+    INSTALL_ENUM(ULocMatchDemotion, "NONE", ULOCMATCH_DEMOTION_NONE);
+    INSTALL_ENUM(ULocMatchDemotion, "REGION", ULOCMATCH_DEMOTION_REGION);
+#endif
 }
