@@ -1,5 +1,5 @@
 /* ====================================================================
- * Copyright (c) 2004-2018 Open Source Applications Foundation.
+ * Copyright (c) 2004-2019 Open Source Applications Foundation.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -51,6 +51,15 @@ PyObject *make_descriptor(PyObject *(*get)(PyObject *));
             return ICUException(status).reportError();                  \
     }
 
+#define STATUS_RESULT_CALL(action, result)                              \
+    {                                                                   \
+        UErrorCode status = U_ZERO_ERROR;                               \
+        action;                                                         \
+        if (U_FAILURE(status))                                          \
+            return ICUException(status).reportError();                  \
+        result;                                                         \
+    }
+
 #define API_STATUS_CALL(var, api, args)                                 \
     {                                                                   \
         UErrorCode status = U_ZERO_ERROR;                               \
@@ -143,6 +152,9 @@ PyObject *make_descriptor(PyObject *(*get)(PyObject *));
 #define DECLARE_METHOD(type, name, flags)                               \
     { #name, (PyCFunction) type##_##name, flags, "" }
 
+#define DECLARE_ABSTRACT_METHOD(type, name, flags)                      \
+    { #name, (PyCFunction) abstract_method, flags, "" }
+
 #define DECLARE_TYPE(name, t_name, base, icuClass, init, dealloc)           \
 PyTypeObject name##Type_ = {                                                \
     PyVarObject_HEAD_INIT(NULL, 0)                                          \
@@ -208,9 +220,9 @@ void t_name##_dealloc(t_name *self)                                   \
     Py_TYPE(self)->tp_free((PyObject *) self);                        \
 }                                                                     \
 DECLARE_TYPE(name, t_name, base, icuClass, init, t_name##_dealloc)    \
-PyObject *wrap_##name(const icuClass &object)                         \
+PyObject *wrap_##name(icuClass object)                                \
 {                                                                     \
-    return wrap_##name(new icuClass(object), T_OWNED);                \
+    return wrap_##name(new icuClass(std::move(object)), T_OWNED);     \
 }
 
 
@@ -415,15 +427,20 @@ PyTypeObject name##Type_ = {                                            \
                 if (op == Py_EQ)                                          \
                     Py_RETURN_BOOL(b);                                    \
                 Py_RETURN_BOOL(!b);                                       \
-              case Py_LT:                                                 \
-              case Py_LE:                                                 \
-              case Py_GT:                                                 \
-              case Py_GE:                                                 \
+              default:                                                    \
                 PyErr_SetNone(PyExc_NotImplementedError);                 \
                 return NULL;                                              \
             }                                                             \
         }                                                                 \
-        return PyErr_SetArgsError((PyObject *) self, "__richcmp__", arg); \
+        switch (op) {                                                     \
+          case Py_EQ:                                                     \
+            Py_RETURN_FALSE;                                              \
+          case Py_NE:                                                     \
+            Py_RETURN_TRUE;                                               \
+          default:                                                        \
+            PyErr_SetNone(PyExc_NotImplementedError);                     \
+            return NULL;                                                  \
+        }                                                                 \
     }
 
 
