@@ -566,6 +566,12 @@ void _init_script(PyObject *m)
     INSTALL_ENUM(UScriptCode, "SOGDIAN", USCRIPT_SOGDIAN);
     INSTALL_ENUM(UScriptCode, "OLD_SOGDIAN", USCRIPT_OLD_SOGDIAN);
 #endif
+#if U_ICU_VERSION_HEX >= VERSION_HEX(64, 0, 0)
+    INSTALL_ENUM(UScriptCode, "ELYMAIC", USCRIPT_ELYMAIC);
+    INSTALL_ENUM(UScriptCode, "NANDINAGARI", USCRIPT_NANDINAGARI);
+    INSTALL_ENUM(UScriptCode, "NYIAKENG_PUACHUE_HMONG", USCRIPT_NYIAKENG_PUACHUE_HMONG);
+    INSTALL_ENUM(UScriptCode, "WANCHO", USCRIPT_WANCHO);
+#endif
 
 #if U_ICU_VERSION_HEX >= VERSION_HEX(51, 0, 0)
     INSTALL_ENUM(UScriptUsage, "NOT_ENCODED", USCRIPT_USAGE_NOT_ENCODED);
